from .pretty_printer import custom_print


def print_courses_data(json_data: dict, settings: dict, title: str = "Your courses:", prefixes: dict = None):
    """
    Print out the courses in json_data in a neat way
    :param json_data: json object with data about Dodona courses
    :param settings: dict with settings
    :param title: title to display above the courses-list
    :param prefixes: dictionary with a prefix for each id in json_data
    """
    if prefixes is None:
        prefixes = {}

    # List of tuples where each tuple represents a course by id, name and teacher
    display_data: list[tuple] = []

    for field in json_data:
        display_data.append((str(field['id']), field['name'], field['teacher']))

    # Find the maximum length of all but the last element in all tuples to align them in the terminal
    max_course_id_length = max(len(e[0]) for e in display_data)
    max_course_name_length = max(len(e[1]) for e in display_data)

    # Print out all courses in display_data
    result = f'[u bright_blue]{title}[/]\n'
    for course in display_data:
        result += (prefixes.get(course[0]) or "\t")
        result += (f"{course[0].ljust(max_course_id_length)}: [bold]"
                   f"{course[1].ljust(max_course_name_length)}[/]\tby {course[2]}\n")

    custom_print(result.strip(), settings, pretty=True)


def print_series_data(json_data: dict, settings: dict, force: bool = False, prefixes: dict = None):
    """
    Print out the exercise-series in json_data in a neat (unless force) way.
    :param json_data: Json object with data about Dodona exercise-series
    :param settings: dict with settings
    :param force: Boolean to decide if the series description has to be printed, or only a link to it
    :param prefixes: Dictionary with a prefix for each id in json_data
    """
    if prefixes is None:
        prefixes = {}

    # List of tuples where each tuple represents an exercise-series by id, name and description
    display_data = []

    for field in json_data:
        display_data.append(
            (
                str(field['id']),
                field['name'].strip(),
                field['description']
            )
        )

    # Find the maximum length of all but the last element in all tuples to align them in the terminal
    max_series_id_length = max(len(e[0]) for e in display_data)
    max_series_name_length = max(len(e[1]) for e in display_data)

    # Print out all the series in display_data while also handling the Markdown inside the series-description
    result = "[u bright_blue]All series:[/]\n"
    if force:
        import markdownify
        import re
        from rich.markdown import Markdown
        from rich.padding import Padding
        from dodonacli.source import pretty_console

        pretty_console.console.print('\n' * settings['new_lines_above'] + result, end='')

        for i, series in enumerate(display_data):
            description = series[2].strip('\n')
            description = re.sub(r'{: *target="_blank"}', '', description).strip()
            md_description = Markdown(markdownify.markdownify(description))

            pretty_console.console.print(
                f"\t{series[0].ljust(max_series_id_length)}: "
                f"[bold]{series[1].ljust(max_series_name_length)}[/]"
            )
            pretty_console.console.print(
                Padding(
                    md_description,
                    pad=(0, 0, 1 if description and i + 1 < len(display_data) else 0, 12)
                ), end=''
            )
        print('\n' * settings['new_lines_below'], end='')

    else:
        for i, series in enumerate(display_data):
            result += prefixes.get(series[0]) or "\t"
            result += f"{series[0].ljust(max_series_id_length)}: [bold]{series[1].ljust(max_series_name_length)}[/]\n"

        custom_print(result.strip(), settings, pretty=True)


def print_exercise_data(json_data: dict, settings: dict, prefixes: dict = None):
    """
    Print out the exercises in json_data in a neat way
    :param json_data: json object with data about Dodona exercises in a series
    :param settings: dict with settings
    :param prefixes: dictionary with a prefix for each id in json_data
    """
    if prefixes is None:
        prefixes = {}

    # List of tuples where each tuple represents an exercise by id, name, solved and has_attempt
    display_data = []

    for field in json_data:
        if field['type'] == "ContentPage":
            display_data.append({
                'type': "ContentPage",
                'id': str(field['id']),
                'name': field['name'],
                'has_read': field['has_read']
            })
        elif field['type'] == "Exercise":
            display_data.append({
                'type': "Exercise",
                'id': str(field['id']),
                'name': field['name'],
                # 'last_solution_is_best': field['last_solution_is_best'],
                'has_solution': field['has_solution'],
                # 'has_correct_solution': field['has_correct_solution'],
                'accepted': field['accepted']
            })

    # Find the maximum length of all but the last element in all tuples to align them in the terminal
    max_exercise_id_length = max(len(e['id']) for e in display_data)
    max_exercise_name_length = max(len(e['name']) for e in display_data)

    # Print out all exercises in display_data with indicator about solution-status: solved, wrong or not yet solved
    result = '[u bright_blue]Exercises:[/]\n'
    for exercise in display_data:
        if exercise['type'] == "Exercise":
            if exercise['accepted']:
                solve_status = "[bold bright_green]SOLVED[/]"
            elif exercise['has_solution']:
                solve_status = "[bold bright_red]WRONG[/]"
            else:
                solve_status = "[bold]NOT YET SOLVED[/]"

        elif exercise['type'] == "ContentPage":
            if exercise['has_read']:
                solve_status = "[bold bright_green]READ[/]"
            else:
                solve_status = "[bold]NOT YET READ[/]"

        else:
            solve_status = "[bold]SOLVE STATUS UNKNOWN"

        result += (prefixes.get(exercise['id']) or "\t")
        result += f"{exercise['id'].ljust(max_exercise_id_length)}: "
        result += f"[bold]{exercise['name'].ljust(max_exercise_name_length)}[/]\t" + solve_status + '\n'

    custom_print(result.strip(), settings, pretty=True)


def print_exercise(json_data: dict, token: str, settings: dict, force: bool = False):
    """
    Print out the exercise-description.
    Needs to call the Dodona sandbox and convert HTML to text.
    Print out a warning for potential incompleteness, which may be dangerous for tests and exams.
    :param token: API-token as authorization
    :param json_data: json object with info about a Dodona exercise
    :param force: boolean to decide if the exercise description has to be printed, or only a link to it
    :param settings: dict with settings
    """
    if json_data['type'] == 'ContentPage':
        custom_print(
            "No need to program anything this time, but you'll have to go read this and mark it as read:\n"
            + json_data['url'].replace(".json", ""),
            settings, pretty=True
        )

    elif not force:
        custom_print(
            f"You can find the exercise description at \n{json_data['description_url']}",
            settings,
            pretty=True
        )

    else:
        import http.client
        import markdownify
        from bs4 import BeautifulSoup
        from rich.markdown import Markdown
        from rich.padding import Padding
        from dodonacli.source import get_data, pretty_console

        custom_print(
            "Expected programming language: " + json_data['programming_language']['name'] + '\n',
            {'new_lines_above': settings['new_lines_above']},
            pretty=True
        )

        # Make sandbox.dodona connection for exercise description:
        sandbox = http.client.HTTPSConnection("sandbox.dodona.be")
        headers = {"Authorization": token}

        stripped_link = json_data['description_url'].replace("https://sandbox.dodona.be", "", 1)
        sandbox.request("GET", stripped_link, headers=headers)

        data = get_data.handle_connection_response(sandbox).decode()

        soup = BeautifulSoup(data, features="html.parser")
        html_description = str(soup.find("div", {"class": "card-supporting-text"}))

        md_description = markdownify.markdownify(html_description).strip()

        md = Markdown(md_description)

        if settings['paste_force_warning']:
            # Print the HTML with warnings
            warning = (
                    "\n[u bold bright_red]WARNING:[/] the description may be incorrect, "
                    "DO NOT rely on this for exams and tests!\n"
                    "View in browser: " + json_data['description_url'] + '\n'
            )
            pretty_console.console.print(warning)
            pretty_console.console.print(Padding(md, pad=(0, 0, 0, 3)))
            pretty_console.console.print(warning)
        else:
            pretty_console.console.print(Padding(md, pad=(0, 0, 0, 3)))

        print('\n' * settings['new_lines_below'], end='')


def print_result(json_results: dict, url: str, settings: dict):
    """
    Print out the results of a submission in a neat way
    :param json_results: json object with data about a submission
    :param url: link to the submission
    :param settings: dict with settings
    """
    from dodonacli.source import submission_data_handler

    if json_results['accepted']:
        # Everything passed, well done!
        result = "[bold bright_green]All tests passed![/] You can continue to next exercise.\n"
    else:
        result = submission_data_handler.submission_data_handler(json_results, settings).strip() + '\n'

    result += url
    custom_print(result, settings, pretty=True)


def print_status(config: dict, settings: dict):
    """
    Print out the current selection of course, exercise-series and exercise.
    :param config: Dictionary with the configs
    :param settings: Dictionary with the settings
    """
    course_string = config['course_name']
    if config['course_id'] is not None:
        course_string += f" ({config['course_id']})"

    custom_print(
        f"[u bright_blue]Status:[/]\n"
        f"\t{'Course: '.ljust(10)}{course_string}\n"
        f"\t{'Series: '.ljust(10)}{config['serie_name']}\n"
        f"\t{'Exercise: '.ljust(10)}{config['exercise_name']}",
        settings, pretty=True
    )


def print_exercise_submissions(json_data: dict, settings: dict):
    """
    Print out a list of the (up to) 10 most recent submissions as found in json_data
    :param json_data: Dictionary with submission data
    :param settings: Dictionary with settings
    """
    from dodonacli.source import pretty_console

    pretty_console.console.print(
        "\n[u bright_blue]Most recent submissions:[/]"
    )

    amount_shown = min(settings['amount_sub_exercise'], len(json_data))
    if amount_shown == -1:
        amount_shown = len(json_data)

    for i, submission in enumerate(json_data[:amount_shown]):
        if submission['accepted']:
            accepted_emoji = "[bright_green]:heavy_check_mark:[/bright_green]"
        else:
            accepted_emoji = "[bright_red]:heavy_multiplication_x:[/bright_red]"

        status = submission['status']
        if submission['status'] in ("memory limit exceeded", "geheugenlimiet overschreden"):
            status += "\n\t\t\tWow, how did you do that?"

        pretty_console.console.print(
            f"\t{accepted_emoji}  [link={submission['url'].rstrip('.json')}]#{len(json_data) - i: <2}[/link]"
            f"\t{status}\t"
        )
    # Newline for clarity
    print()


def print_all_submissions(connection, headers: dict, json_data: dict, settings: dict):
    """
    Print out a list of the latest 30 submissions for the user, userwide (not tied to an exercise).
    Makes extra requests to Dodona to get the name of the exercises of the submissions
    :param connection: Connection to Dodona
    :param headers: Headers to send with the connection
    :param json_data: Dictionary with submission info
    :param settings: Dictionary with settings
    :return:
    """
    from dodonacli.source import pretty_console
    import json

    print('\n' * settings['new_lines_above'], end='')

    pretty_console.console.print(
        "[u bright_blue]Most recent submissions:[/]"
    )

    amount_shown = min(settings['amount_sub_global'], len(json_data))
    if amount_shown == -1:
        amount_shown = len(json_data)

    for i, submission in enumerate(json_data[:amount_shown]):
        if submission['accepted']:
            accepted_emoji = "[bright_green]:heavy_check_mark:[/bright_green]"
        else:
            accepted_emoji = "[bright_red]:heavy_multiplication_x:[/bright_red]"

        status = submission['status']

        exercise_link = submission['exercise'].replace("https://dodona.be/nl", "", 1)
        connection.request("GET", exercise_link, headers=headers)

        response = connection.getresponse()
        res_status = response.status
        # Need to read always to be able to accept a new request
        result = response.read()

        if res_status == 200:
            exercise_name = json.loads(result).get('name') or "[i]unkown[/i]"
        else:
            exercise_name = "[i]unable to get info about exercise[/i]"

        pretty_console.console.print(
            f"\t{accepted_emoji}  [link={submission['url'].rstrip('.json')}]#{len(json_data) - i: <2}[/link]"
            f"\t{status: <25}"
            f"\t{exercise_name}"
        )

    connection.close()
    print('\n' * settings['new_lines_below'], end='')



import click
from click_default_group import DefaultGroup


@click.group(help="Get submission data. Default = view",
             cls=DefaultGroup, default='view', default_if_no_args=True)
def sub():
    pass


@click.command(help="Load a submission to the prev_submission file, and display more info about it. "
                    "You can specify a number, else it takes the last submission for that exercise.")
@click.argument('number',
                type=click.IntRange(min=0), default=0)
def load(number):
    import http.client
    from dodonacli.source import get_data, set_data

    # Read configs in
    config = get_data.get_configs()

    # Start up the connection to Dodona
    connection = http.client.HTTPSConnection("dodona.be")
    headers = {
        "Content-type": "application/json",
        "Accept": "application/json",
        "Authorization": config['TOKEN']
    }

    if config['exercise_id']:
        json_all_submissions = get_data.exercise_submissions(config, connection, headers)
        extension = "." + get_data.get_extension(config['programming_language']) or ""
    else:
        json_all_submissions = get_data.all_submissions(connection, headers)
        extension = ""

    submission = json_all_submissions[-int(number)]

    submission_info = get_data.submission_info(submission['id'], connection, headers, config)
    set_data.save_to_file(
        submission_info['exercise_name'], submission_info['id'], submission_info['code'],
        get_data.get_settings(), extension
    )

    return


@click.command(help="View subimmision data")
def view():
    import http.client
    from dodonacli.source import get_data, pretty_print

    # Read configs in
    config = get_data.get_configs()
    settings = get_data.get_settings()

    # Start up the connection to Dodona
    connection = http.client.HTTPSConnection("dodona.be")
    headers = {
        "Content-type": "application/json",
        "Accept": "application/json",
        "Authorization": config['TOKEN']
    }

    if config['exercise_id']:
        json_data = get_data.exercise_submissions(config, connection, headers)
        pretty_print.print_exercise_submissions(json_data, settings)
    else:
        json_data = get_data.all_submissions(connection, headers)
        pretty_print.print_all_submissions(connection, headers, json_data, settings)


sub.add_command(load)
sub.add_command(view)



import click


@click.group(help="Info about shell-completion, changelog, version, "
                  "update-availability and GitHub page.")
def info():
    pass


@click.command(help='Display the current version of DodonaCLI. The versioning system '
                    'uses a YYYY.M.D format.')
def version():
    from dodonacli.source import get_data, pretty_printer

    pretty_printer.custom_print("DodonaCLI " + get_dodonacli_version(), get_data.get_settings(), pretty=True)


@click.command(help='Checks if there is a new update available for DodonaCLI.')
def check_update():
    from packaging.version import parse
    from pkg_info import get_pkg_info
    from dodonacli.source import get_data, pretty_printer

    settings = get_data.get_settings()
    dodonacli_version = get_dodonacli_version()

    pkg = get_pkg_info('DodonaCLI')

    if parse(pkg.version) > parse(dodonacli_version):
        text = f"There is a new version available: {pkg.version}\n" \
               f"You can update your old version ({dodonacli_version}) with\n" \
               f"  'pip install --upgrade DodonaCLI'"
    else:
        text = "Your DodonaCLI is up-to-date."

    pretty_printer.custom_print(text, settings, pretty=True)


@click.command(help='Tab completion, very handy for fast use')
def completion():
    from dodonacli.source import get_data, pretty_printer

    settings = get_data.get_settings()

    pretty_printer.custom_print(
        "There are 2 ways of doing tab-completion: \n"
        "   - using Click's default tab-completion for bash/zsh/fish\n"
        "   - using DodonaCLI's custom script for bash only\n"
        "\nThe reason there is a custom script, is because the default completion lacks "
        "a bit here and there.\n"
        "The default option is easier to use, and doesn't need a redownload after an update.\n\n"
        "To install the default completion:\n"
        "   Follow this short tutorial:\n"
        "       https://click.palletsprojects.com/en/8.1.x/shell-completion/#enabling-completion\n"
        "   Replace every occurence of 'foo-bar' with 'dodona'\n"
        "   Do notice that you can choose how you name the file, and where you put it.\n\n"
        "To install the custom script for bash:\n"
        "   Go to DodonaCLI's GitHub: https://www.github.com/BWindey/DodonaCLI\n"
        "   And download 'dodonacli_completion_script.sh' (at top-level of project structure)\n"
        "   Now add 'source <PATH TO SCRIPT>' to your '.bashrc',\n"
        "   where you fill in the path to the downloaded script."
        "\n\n"
        "For both ways you'll have to either [u yellow]restart your terminal[/], "
        "or re-'source' your .bashrc/.fishrc/...\n"
        "Happy tabbing!",
        settings, pretty=True
    )


@click.command(help="Link to the GitHub page of DodonaCLI. "
                    "Can be handy for the README page, manpages, Issues (bug reports) and pull requests.")
def github():
    from dodonacli.source import get_data, pretty_printer

    settings = get_data.get_settings()

    pretty_printer.custom_print(
        "https://www.github.com/BWindey/DodonaCLI",
        settings, pretty=True
    )


@click.command(help='Changelog for the latest version.')
def changelog():
    from rich.markdown import Markdown
    from dodonacli.source import get_data, pretty_console

    settings = get_data.get_settings()

    changelog_raw = (
        "\t- Added some forgotten settings-driven prints, I think every print now takes the "
        "new-lines into account.\n"
        "\t- Moved some imports around, speeding up some subcommands that don't connect to Dodona.\n"
        "\nThis was a small quick update, so here is the changelog for the previous update:\n\n"
        "\t- Added man-page sub-command to completion-script, oopsie,"
        "forgot that previously\n"
        "\t- Added a settings.json file at the same location as config.json. "
        "This settings file may be edited directly by the user, the config one NOT.\n"
        "\t- All the implemented settings, followed by their default setting:\n"
        "\t\t- 'amount_feedback_context': 3\n"
        "\t\t- 'amount_feedback_tab': -1\n"
        "\t\t- 'amount_feedback_testcase': 3\n"
        "\t\t- 'amount_feedback_test': 3\n"
        "\t\t- 'amount_sub_exercise': 10\n"
        "\t\t- 'amount_sub_global': -1\n"
        "\t\t- 'new_lines_above': 1\n"
        "\t\t- 'new_lines_below': 1\n"
        "\t\t- 'paste_force_warning': True\n"
        "\t\t- 'display_series_after_select': False\n"
        "\t\t- 'display_exercises_after_select': False\n"
        "\t\t- 'display_exercise_after_select': False\n\n"
        "\t\t* For feedback-settings, the order is tab->context->testcase->test. The number "
        "tells you how many of them will be shown in your terminal.\n"
        "\t\t* The new-lines ones describe how many empty lines should be printed around "
        "each complete message DodonaCLI prints to your terminal. This has always be 1-1, "
        "but is now customisable.\n"
        "\t\t* The display ... after select does exactly what you think it does!\n"
        "\t\t* An integer set to -1 means that there is no upper bound but that "
        "of the amount of data the Dodona API gives.\n\n"
        "\t- Drastically improved the 'display --force' when only a course is selected\n"
        "\n"
        "\tAs always, you can use the '--help' flag after every command and sub-command to learn more.\n"
        "\tHappy coding!"
    )
    md = Markdown(changelog_raw)
    print('\n' * settings['new_lines_above'], end='')
    pretty_console.console.print(md)
    print('\n' * settings['new_lines_below'], end='')


@click.command(help='Man-pages for DodonaCLI, very professional')
def man_page():
    from dodonacli.source import get_data, pretty_printer

    settings = get_data.get_settings()

    pretty_printer.custom_print(
        "To install a man-page for DodonaCLI, you can download it from GitHub:\n"
        "  https://www.github.com/BWindey/DodonaCLI\n"
        "The man-page files are located under the 'man-page' directory.\n"
        "Download the .gz version (gzip-compressed), "
        "the other one is not useful unless you want to see how man-pages are written.\n\n"
        "Now run the command 'manpath' in your terminal, this should print a list of "
        "paths, seperated by a ':'\n"
        "Now move the downloaded file to one of those paths.\n"
        "I personally put it onder '~/.local/share/man/man1/'\n"
        "[yellow]Note:[/] the manual-entry is for 'dodonacli', not 'dodona'",
        settings, pretty=True
    )


def get_dodonacli_version():
    from importlib import metadata
    return metadata.version(__package__.split('.')[0])


info.add_command(changelog)
info.add_command(check_update)
info.add_command(completion)
info.add_command(github)
info.add_command(man_page)
info.add_command(version)



import click
import os

from dodonacli.source import pretty_console


def clear_screen():
    os.system('cls' if os.name == 'nt' else 'clear')


def one():
    print("you have selected menu option one") # Simulate function output.
    input("Press Enter to Continue\n")
    clear_screen()


def reset_to_default():
    pass


def done():
    print("Goodbye")
    sys.exit()


@click.command(help="Some user settings")
def settings():
    clear_screen()

    menu_items = {
        '1': one,
        'reset': reset_to_default,
        'done': done
    }

    while True:
        pretty_console.console.print(
            '\n'.join(key + ': ' + value.replace('_', ' ') for key, value in menu_items.items())
        )
        selection = input("Setting: ")
        if selection in menu_items:
            selected_value = menu_items[selection]  # Gets the function name
            selected_value()


if __name__ == "__main__":
    settings()



import os
import shutil
import subprocess
import tempfile


def check_syntax(file: str, language: str) -> bool:
    """
    Check the syntax of the code in the file using external tools
    :param file: Filename containing the code to check
    :param language: Programming language to determine what tool to use
    :return: Returns whether the syntax is right
    """
    if language in ("bash", "sh"):
        return check_bash_syntax(file)
    if language == "python":
        return check_python_syntax(file)
    if language == "java":
        return check_java_syntax(file)
    if language == "javascript":
        return check_javascript_syntax(file)
    print("\nLanguage syntax check is not implemented for: " + language)
    return True


def check_bash_syntax(file: str) -> bool:
    try:
        # Exclude:
        # SC2001: See if you can use ${variable//search/replace} instead.
        subprocess.run(['shellcheck', '-e', 'SC2001', file], check=True)
        return True
    except subprocess.CalledProcessError as cpe:
        print(f"Syntax Error in Bash script: {str(cpe)}")
        return False
    except FileNotFoundError:
        # This will only occur if shellcheck isn't installed.
        # Click will detect that the user gave an invalid file before this function is called
        print(
            "\nTo check the syntax, 'shellcheck' is called with your file. "
            "It appears however, that shellcheck isn't installed on your system. \n"
            "Please install it: https://www.shellcheck.net/\n"
        )
        return False
    except Exception as e:
        print("\nNo idea what's going wrong, but something definitly is going wrong:\n" + str(e))
        return False


def check_python_syntax(file: str) -> bool:
    try:
        with open(file, 'r') as content:
            compile(content.read(), file, 'exec')
        return True
    except SyntaxError as se:
        print(f"\nSyntax error in Python file:\n{se}\n")
    except Exception as e:
        print("\nNo idea what's going wrong, but something definitly is going wrong:\n" + str(e))
        return False


def check_java_syntax(file: str) -> bool:
    temp_dir = tempfile.mkdtemp()
    try:
        directory = os.path.dirname(file)
        java_files = [file for file in os.listdir(directory) if file.endswith('.java') and "test" not in file.lower()]
        if not java_files:
            return False  # No Java files found in the directory

        # Construct the list of Java files with their full paths
        java_files_with_paths = [os.path.join(directory, file) for file in java_files]

        subprocess.run(['javac', '-d', temp_dir] + java_files_with_paths, check=True)

        # Remove temporary directory
        shutil.rmtree(temp_dir, ignore_errors=True)
        return True

    except subprocess.CalledProcessError:
        # This error will happen if there was something wrong with the Java syntax
        return False

    except FileNotFoundError:
        # This will only occur if javac isn't installed.
        # Click will detect that the user gave an invalid file before this function is called
        print(
            "\nTo check the syntax, 'javac' is called with your file. "
            "It appears however, that the Java compiler isn't installed on your system.\n"
            "Please install it: https://openjdk.org/install/\n"
        )
        return False
    except Exception as e:
        print("\nNo idea what's going wrong, but something definitly is going wrong:\n" + str(e))
        return False
    finally:
        shutil.rmtree(temp_dir, ignore_errors=True)


def check_javascript_syntax(file: str) -> bool:
    try:
        subprocess.run(['node', '-c', file], check=True)
        return True
    except subprocess.CalledProcessError:
        return False
    except FileNotFoundError:
        # This will only occur if jshint isn't installed.
        # Click will detect that the user gave an invalid file before this function is called
        print(
            "\nTo check the syntax, 'node' is called with your file. "
            "It appears however, that this program isn't installed on your system.\n"
            "Please install it: https://nodejs.org/en/learn/getting-started/how-to-install-nodejs\n"
        )
        return False
    except Exception as e:
        print("\nNo idea what's going wrong, but something definitly is going wrong:\n" + str(e))
        return False


def check_haskell_syntax(file: str) -> bool:
    try:
        # TODO: enable compiling without main function present
        subprocess.run(['ghc', '-fno-code', '-v0', file], check=True)
        return True
    except subprocess.CalledProcessError:
        return False
    except FileNotFoundError:
        print(
            "\nTo check the syntax, 'ghc' is called with your file. "
            "It appears however, that this program isn't installed on your system.\n"
            "Please install it: https://www.haskell.org/ghcup/\n"
        )
        return False
    except Exception as e:
        print("\nNo idea what's going wrong, but something definitly is going wrong:\n" + str(e))
        return False



import click


@click.command(help="Post a solution-file to Dodona. "
                    "The file has to be in your current working directory, and this only works "
                    "if there is a selected exercise. "
                    "DodonaCLI will give a try at rendering a small part of the feedback-table so you can continue "
                    "working from the terminal, this isn't guarenteed to provide enough info, but it's doing its best.")
@click.option("-l", "--use-link",
              help="Post your solutionfile to the link at the first line of your solutionfile. "
                   "This is inspired by plugins for editors as VSCode for Dodona.",
              is_flag=True, default=False)
@click.option("-c", "--check",
              help="Check the file you provided if the syntax is valid for the programming language "
                   "associated with the exercise. Currently supported languages: bash, python, java, "
                   "javascript. For Java files, it will use javac to compile all *.java files that "
                   "don't have 'test' in them.",
              is_flag=True, default=False)
@click.argument('file', type=click.Path(exists=True, file_okay=True, dir_okay=False, resolve_path=True))
def post(file, use_link, check):
    import http.client
    from dodonacli.source import set_data, get_data, syntax_checker

    # Read configs in
    config = get_data.get_configs()
    settings = get_data.get_settings()

    # Quick check if combination of selected exercise and -l flag are valid
    if not use_link and not config['exercise_id']:
        print(
            '\n' * settings['new_lines_above']
            + "No exercise selected! If you want to use a link at the top of your file, use the -l flag."
            + '\n' * settings['new_lines_below']
        )

    # Start up the connection to Dodona
    connection = http.client.HTTPSConnection("dodona.be")
    headers = {
        "Content-type": "application/json",
        "Accept": "application/json",
        "Authorization": config['TOKEN']
    }

    # Check for the link at the top of the file
    if use_link:
        with open(file, 'r') as solutionfile:
            link = solutionfile.readline()

            # If file starts with hashbang, look at 2nd line
            if link[:2].strip() == '#!':
                hashbang = link
                link = solutionfile.readline()

            # Check if valid link to try to post to
            link_index = link.find("https://dodona.be/")
            if link_index < 0:
                print(
                    '\n' * settings['new_lines_above']
                    + "No valid link found on the first line of your file. Please confirm again.\n"
                      "A valid link starts with 'https://dodona.be/'"
                    + '\n' * settings['new_lines_below']
                )
                return
            link = link[link_index:]

            # Take out the id's we need
            if link.find("/courses/") != -1:
                course_id_index_start = link.find("/courses/") + len("/courses/")
                course_id_index_stop = link.find("/", course_id_index_start)
                course_id = link[course_id_index_start:course_id_index_stop]
            else:
                course_id = None

            exercise_id_index_start = link.find("/activities/") + len("/activities/")
            exercise_id_index_stop = link.find("/", exercise_id_index_start)
            exercise_id = link[exercise_id_index_start:exercise_id_index_stop]

            # We can read the content of the file now as the file-pointer is already at the 2nd line
            if hashbang:
                content = hashbang + solutionfile.read()
            else:
                content = solutionfile.read()

    else:
        # Syntax check only available without link at top of file
        if check:
            if not syntax_checker.check_syntax(file, config['programming_language']):
                return

        with open(file, 'r') as solutionfile:
            content = solutionfile.read()

        course_id = config['course_id']
        exercise_id = config['exercise_id']

    # Make sure the amount of newlines is exactly 1 to make Dodona's linters happy
    content = content.rstrip() + "\n"

    # Post exercise to Dodona
    set_data.post_solution(content, connection, headers, course_id, exercise_id, settings)
